"""Conversation state persistence for Claude Code SDK.

This module provides comprehensive state persistence capabilities:
- Save and restore conversation history
- Multiple storage backends (file, memory, database)
- Conversation snapshots and checkpoints
- Session management and recovery
- Automatic state synchronization
"""

import json
import pickle
import sqlite3
from abc import ABC, abstractmethod
from contextlib import asynccontextmanager
from dataclasses import asdict, dataclass, field
from datetime import datetime
from pathlib import Path
from typing import Any, AsyncIterator, Protocol

import aiofiles
import anyio
from typing_extensions import TypedDict

from .types import AssistantMessage, Message, ResultMessage, SystemMessage, UserMessage


class StorageBackend(Protocol):
    """Protocol for storage backends."""

    async def save(self, key: str, data: dict[str, Any]) -> None:
        """Save data with key."""
        ...

    async def load(self, key: str) -> dict[str, Any] | None:
        """Load data by key."""
        ...

    async def delete(self, key: str) -> None:
        """Delete data by key."""
        ...

    async def list_keys(self, prefix: str = "") -> list[str]:
        """List all keys with optional prefix."""
        ...

    async def exists(self, key: str) -> bool:
        """Check if key exists."""
        ...


@dataclass
class ConversationState:
    """Represents the state of a conversation.
    
    Example:
        ```python
        state = ConversationState(
            session_id="session-123",
            messages=[UserMessage(content="Hello")],
            metadata={"user_id": "user-456"}
        )
        ```
    """

    session_id: str
    messages: list[Message] = field(default_factory=list)
    created_at: datetime = field(default_factory=datetime.now)
    updated_at: datetime = field(default_factory=datetime.now)
    metadata: dict[str, Any] = field(default_factory=dict)
    checkpoints: list["ConversationCheckpoint"] = field(default_factory=list)
    total_cost: float = 0.0
    total_tokens: int = 0
    model: str | None = None
    options: dict[str, Any] = field(default_factory=dict)

    def add_message(self, message: Message) -> None:
        """Add a message and update timestamp."""
        self.messages.append(message)
        self.updated_at = datetime.now()

    def create_checkpoint(self, name: str | None = None) -> "ConversationCheckpoint":
        """Create a checkpoint of current state."""
        checkpoint = ConversationCheckpoint(
            name=name or f"checkpoint-{len(self.checkpoints) + 1}",
            message_count=len(self.messages),
            timestamp=datetime.now(),
            total_cost=self.total_cost,
            total_tokens=self.total_tokens,
        )
        self.checkpoints.append(checkpoint)
        return checkpoint

    def restore_checkpoint(self, checkpoint: "ConversationCheckpoint") -> None:
        """Restore state to a checkpoint."""
        self.messages = self.messages[: checkpoint.message_count]
        self.total_cost = checkpoint.total_cost
        self.total_tokens = checkpoint.total_tokens
        self.updated_at = datetime.now()

    def to_dict(self) -> dict[str, Any]:
        """Convert to dictionary for serialization."""
        return {
            "session_id": self.session_id,
            "messages": [self._message_to_dict(msg) for msg in self.messages],
            "created_at": self.created_at.isoformat(),
            "updated_at": self.updated_at.isoformat(),
            "metadata": self.metadata,
            "checkpoints": [asdict(cp) for cp in self.checkpoints],
            "total_cost": self.total_cost,
            "total_tokens": self.total_tokens,
            "model": self.model,
            "options": self.options,
        }

    @staticmethod
    def _message_to_dict(message: Message) -> dict[str, Any]:
        """Convert message to dictionary."""
        if isinstance(message, UserMessage):
            return {"type": "user", "content": message.content}
        elif isinstance(message, AssistantMessage):
            return {
                "type": "assistant",
                "content": [asdict(block) for block in message.content],
            }
        elif isinstance(message, SystemMessage):
            return {
                "type": "system",
                "subtype": message.subtype,
                "data": message.data,
            }
        elif isinstance(message, ResultMessage):
            return {
                "type": "result",
                "subtype": message.subtype,
                "cost_usd": message.cost_usd,
                "duration_ms": message.duration_ms,
                "session_id": message.session_id,
                "total_cost_usd": message.total_cost_usd,
                "usage": message.usage,
                "num_turns": message.num_turns,
            }
        else:
            return {"type": "unknown", "data": str(message)}

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> "ConversationState":
        """Create from dictionary."""
        # Convert messages back
        messages = []
        for msg_data in data.get("messages", []):
            msg_type = msg_data.get("type")
            if msg_type == "user":
                messages.append(UserMessage(content=msg_data["content"]))
            # Simplified for brevity - would need full message reconstruction
        
        return cls(
            session_id=data["session_id"],
            messages=messages,
            created_at=datetime.fromisoformat(data["created_at"]),
            updated_at=datetime.fromisoformat(data["updated_at"]),
            metadata=data.get("metadata", {}),
            checkpoints=[
                ConversationCheckpoint(**cp) for cp in data.get("checkpoints", [])
            ],
            total_cost=data.get("total_cost", 0.0),
            total_tokens=data.get("total_tokens", 0),
            model=data.get("model"),
            options=data.get("options", {}),
        )


@dataclass
class ConversationCheckpoint:
    """A checkpoint in conversation history."""

    name: str
    message_count: int
    timestamp: datetime
    total_cost: float
    total_tokens: int
    metadata: dict[str, Any] = field(default_factory=dict)


class FileStorageBackend:
    """File-based storage backend.
    
    Example:
        ```python
        backend = FileStorageBackend(Path("./conversations"))
        await backend.save("session-123", state_dict)
        ```
    """

    def __init__(self, base_path: Path, format: str = "json"):
        """Initialize file storage.
        
        Args:
            base_path: Base directory for storage
            format: Storage format ("json" or "pickle")
        """
        self.base_path = base_path
        self.format = format
        self.base_path.mkdir(parents=True, exist_ok=True)

    def _get_file_path(self, key: str) -> Path:
        """Get file path for key."""
        extension = "json" if self.format == "json" else "pkl"
        return self.base_path / f"{key}.{extension}"

    async def save(self, key: str, data: dict[str, Any]) -> None:
        """Save data to file."""
        file_path = self._get_file_path(key)
        
        if self.format == "json":
            async with aiofiles.open(file_path, "w") as f:
                await f.write(json.dumps(data, indent=2))
        else:
            async with aiofiles.open(file_path, "wb") as f:
                await f.write(pickle.dumps(data))

    async def load(self, key: str) -> dict[str, Any] | None:
        """Load data from file."""
        file_path = self._get_file_path(key)
        
        if not file_path.exists():
            return None
        
        if self.format == "json":
            async with aiofiles.open(file_path, "r") as f:
                content = await f.read()
                return json.loads(content)
        else:
            async with aiofiles.open(file_path, "rb") as f:
                content = await f.read()
                return pickle.loads(content)

    async def delete(self, key: str) -> None:
        """Delete file."""
        file_path = self._get_file_path(key)
        if file_path.exists():
            file_path.unlink()

    async def list_keys(self, prefix: str = "") -> list[str]:
        """List all keys."""
        extension = "json" if self.format == "json" else "pkl"
        pattern = f"{prefix}*.{extension}" if prefix else f"*.{extension}"
        
        keys = []
        for file_path in self.base_path.glob(pattern):
            key = file_path.stem
            keys.append(key)
        
        return sorted(keys)

    async def exists(self, key: str) -> bool:
        """Check if key exists."""
        return self._get_file_path(key).exists()


class MemoryStorageBackend:
    """In-memory storage backend for testing and caching.
    
    Example:
        ```python
        backend = MemoryStorageBackend()
        await backend.save("key", {"data": "value"})
        ```
    """

    def __init__(self):
        self._store: dict[str, dict[str, Any]] = {}

    async def save(self, key: str, data: dict[str, Any]) -> None:
        """Save to memory."""
        self._store[key] = data.copy()

    async def load(self, key: str) -> dict[str, Any] | None:
        """Load from memory."""
        return self._store.get(key, {}).copy() if key in self._store else None

    async def delete(self, key: str) -> None:
        """Delete from memory."""
        self._store.pop(key, None)

    async def list_keys(self, prefix: str = "") -> list[str]:
        """List keys."""
        if prefix:
            return sorted([k for k in self._store if k.startswith(prefix)])
        return sorted(self._store.keys())

    async def exists(self, key: str) -> bool:
        """Check existence."""
        return key in self._store


class SQLiteStorageBackend:
    """SQLite-based storage backend.
    
    Example:
        ```python
        backend = SQLiteStorageBackend("conversations.db")
        await backend.initialize()
        await backend.save("session-123", state_dict)
        ```
    """

    def __init__(self, db_path: str | Path):
        self.db_path = str(db_path)
        self._initialized = False

    async def initialize(self) -> None:
        """Initialize database schema."""
        def _init_db():
            conn = sqlite3.connect(self.db_path)
            conn.execute("""
                CREATE TABLE IF NOT EXISTS conversations (
                    key TEXT PRIMARY KEY,
                    data TEXT NOT NULL,
                    created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                    updated_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
                )
            """)
            conn.execute("""
                CREATE INDEX IF NOT EXISTS idx_created_at 
                ON conversations(created_at)
            """)
            conn.commit()
            conn.close()
        
        await anyio.to_thread.run_sync(_init_db)
        self._initialized = True

    async def save(self, key: str, data: dict[str, Any]) -> None:
        """Save to database."""
        if not self._initialized:
            await self.initialize()
        
        def _save():
            conn = sqlite3.connect(self.db_path)
            conn.execute("""
                INSERT OR REPLACE INTO conversations (key, data, updated_at)
                VALUES (?, ?, CURRENT_TIMESTAMP)
            """, (key, json.dumps(data)))
            conn.commit()
            conn.close()
        
        await anyio.to_thread.run_sync(_save)

    async def load(self, key: str) -> dict[str, Any] | None:
        """Load from database."""
        if not self._initialized:
            await self.initialize()
        
        def _load():
            conn = sqlite3.connect(self.db_path)
            cursor = conn.execute(
                "SELECT data FROM conversations WHERE key = ?", (key,)
            )
            row = cursor.fetchone()
            conn.close()
            return json.loads(row[0]) if row else None
        
        return await anyio.to_thread.run_sync(_load)

    async def delete(self, key: str) -> None:
        """Delete from database."""
        if not self._initialized:
            await self.initialize()
        
        def _delete():
            conn = sqlite3.connect(self.db_path)
            conn.execute("DELETE FROM conversations WHERE key = ?", (key,))
            conn.commit()
            conn.close()
        
        await anyio.to_thread.run_sync(_delete)

    async def list_keys(self, prefix: str = "") -> list[str]:
        """List keys."""
        if not self._initialized:
            await self.initialize()
        
        def _list():
            conn = sqlite3.connect(self.db_path)
            if prefix:
                cursor = conn.execute(
                    "SELECT key FROM conversations WHERE key LIKE ? ORDER BY key",
                    (f"{prefix}%",)
                )
            else:
                cursor = conn.execute(
                    "SELECT key FROM conversations ORDER BY key"
                )
            keys = [row[0] for row in cursor.fetchall()]
            conn.close()
            return keys
        
        return await anyio.to_thread.run_sync(_list)

    async def exists(self, key: str) -> bool:
        """Check existence."""
        if not self._initialized:
            await self.initialize()
        
        def _exists():
            conn = sqlite3.connect(self.db_path)
            cursor = conn.execute(
                "SELECT 1 FROM conversations WHERE key = ? LIMIT 1", (key,)
            )
            exists = cursor.fetchone() is not None
            conn.close()
            return exists
        
        return await anyio.to_thread.run_sync(_exists)


class ConversationPersistence:
    """Main conversation persistence manager.
    
    Example:
        ```python
        import asyncio
        from claude_max.persistence import (
            ConversationPersistence,
            FileStorageBackend
        )
        
        async def main():
            # Create persistence manager
            backend = FileStorageBackend(Path("./conversations"))
            persistence = ConversationPersistence(backend)
            
            # Save conversation
            state = ConversationState(session_id="session-123")
            await persistence.save_conversation(state)
            
            # Load conversation
            loaded = await persistence.load_conversation("session-123")
            
            # List sessions
            sessions = await persistence.list_sessions()
        
        asyncio.run(main())
        ```
    """

    def __init__(
        self,
        backend: StorageBackend,
        auto_save: bool = True,
        save_interval: float = 60.0,
    ):
        """Initialize persistence manager.
        
        Args:
            backend: Storage backend to use
            auto_save: Enable automatic saving
            save_interval: Auto-save interval in seconds
        """
        self.backend = backend
        self.auto_save = auto_save
        self.save_interval = save_interval
        self._active_sessions: dict[str, ConversationState] = {}
        self._save_tasks: dict[str, Any] = {}

    async def save_conversation(self, state: ConversationState) -> None:
        """Save conversation state.
        
        Args:
            state: Conversation state to save
        """
        await self.backend.save(state.session_id, state.to_dict())
        self._active_sessions[state.session_id] = state

    async def load_conversation(self, session_id: str) -> ConversationState | None:
        """Load conversation state.
        
        Args:
            session_id: Session ID to load
            
        Returns:
            Loaded conversation state or None
        """
        # Check active sessions first
        if session_id in self._active_sessions:
            return self._active_sessions[session_id]
        
        # Load from backend
        data = await self.backend.load(session_id)
        if data:
            state = ConversationState.from_dict(data)
            self._active_sessions[session_id] = state
            return state
        
        return None

    async def delete_conversation(self, session_id: str) -> None:
        """Delete conversation.
        
        Args:
            session_id: Session ID to delete
        """
        await self.backend.delete(session_id)
        self._active_sessions.pop(session_id, None)
        
        # Cancel auto-save task if exists
        if session_id in self._save_tasks:
            self._save_tasks[session_id].cancel()
            del self._save_tasks[session_id]

    async def list_sessions(self, prefix: str = "") -> list[str]:
        """List all session IDs.
        
        Args:
            prefix: Optional prefix filter
            
        Returns:
            List of session IDs
        """
        return await self.backend.list_keys(prefix)

    async def exists(self, session_id: str) -> bool:
        """Check if session exists.
        
        Args:
            session_id: Session ID to check
            
        Returns:
            True if exists
        """
        return session_id in self._active_sessions or await self.backend.exists(session_id)

    @asynccontextmanager
    async def session(self, session_id: str) -> AsyncIterator[ConversationState]:
        """Context manager for working with a session.
        
        Example:
            ```python
            async with persistence.session("session-123") as state:
                state.add_message(UserMessage(content="Hello"))
                # Auto-saves on exit
            ```
        """
        # Load or create session
        state = await self.load_conversation(session_id)
        if not state:
            state = ConversationState(session_id=session_id)
        
        # Start auto-save if enabled
        if self.auto_save:
            self._start_auto_save(state)
        
        try:
            yield state
        finally:
            # Save on exit
            await self.save_conversation(state)
            
            # Stop auto-save
            if session_id in self._save_tasks:
                self._save_tasks[session_id].cancel()
                del self._save_tasks[session_id]

    def _start_auto_save(self, state: ConversationState) -> None:
        """Start auto-save task for a session."""
        async def auto_save_loop():
            while True:
                await anyio.sleep(self.save_interval)
                await self.save_conversation(state)
        
        # Cancel existing task if any
        if state.session_id in self._save_tasks:
            self._save_tasks[state.session_id].cancel()
        
        # Start new task
        task = anyio.create_task_group().start_soon(auto_save_loop)
        self._save_tasks[state.session_id] = task

    async def create_checkpoint(
        self, session_id: str, name: str | None = None
    ) -> ConversationCheckpoint | None:
        """Create a checkpoint for a session.
        
        Args:
            session_id: Session ID
            name: Optional checkpoint name
            
        Returns:
            Created checkpoint or None
        """
        state = await self.load_conversation(session_id)
        if state:
            checkpoint = state.create_checkpoint(name)
            await self.save_conversation(state)
            return checkpoint
        return None

    async def restore_checkpoint(
        self, session_id: str, checkpoint_name: str
    ) -> bool:
        """Restore session to a checkpoint.
        
        Args:
            session_id: Session ID
            checkpoint_name: Checkpoint name
            
        Returns:
            True if restored successfully
        """
        state = await self.load_conversation(session_id)
        if state:
            for checkpoint in state.checkpoints:
                if checkpoint.name == checkpoint_name:
                    state.restore_checkpoint(checkpoint)
                    await self.save_conversation(state)
                    return True
        return False

    async def export_session(
        self, session_id: str, format: str = "json"
    ) -> str | bytes | None:
        """Export session in specified format.
        
        Args:
            session_id: Session ID to export
            format: Export format ("json", "markdown", etc.)
            
        Returns:
            Exported data or None
        """
        state = await self.load_conversation(session_id)
        if not state:
            return None
        
        if format == "json":
            return json.dumps(state.to_dict(), indent=2)
        elif format == "markdown":
            # Simple markdown export
            lines = [f"# Conversation {session_id}\n"]
            lines.append(f"Created: {state.created_at}\n")
            lines.append(f"Updated: {state.updated_at}\n")
            lines.append("\n## Messages\n")
            
            for msg in state.messages:
                if isinstance(msg, UserMessage):
                    lines.append(f"**User**: {msg.content}\n")
                elif isinstance(msg, AssistantMessage):
                    lines.append("**Assistant**: ")
                    # Simplified - would need proper content rendering
                    lines.append("...\n")
            
            return "\n".join(lines)
        
        return None


def create_persistence(
    storage_type: str = "file",
    path: str | Path = "./conversations",
    auto_save: bool = True,
) -> ConversationPersistence:
    """Create a persistence manager with specified backend.
    
    Args:
        storage_type: Type of storage ("file", "memory", "sqlite")
        path: Path for file-based storage
        auto_save: Enable automatic saving
        
    Returns:
        Configured ConversationPersistence instance
        
    Example:
        ```python
        persistence = create_persistence(
            storage_type="sqlite",
            path="conversations.db"
        )
        ```
    """
    if storage_type == "file":
        backend = FileStorageBackend(Path(path))
    elif storage_type == "memory":
        backend = MemoryStorageBackend()
    elif storage_type == "sqlite":
        backend = SQLiteStorageBackend(path)
    else:
        raise ValueError(f"Unknown storage type: {storage_type}")
    
    return ConversationPersistence(backend, auto_save=auto_save)